<?php
/**
 * @copyright   Copyright (C) Chatbro. All rights reserved.
 * @license     GNU General Public License version 2 or later.
 */

namespace Drupal\chatbro\Library\Chatbroapp\Common;

defined('CHATBROENGINE') || die();

use Drupal\chatbro\Library\Chatbroapp\Core\CbroBackendable;
use Drupal\chatbro\Library\Chatbroapp\Utils\CbroUtils;
use Drupal\chatbro\Library\Chatbroapp\Exceptions\CbroStoreNotFound;

class CbroCommon extends CbroBackendable
{
	private static $tableName = 'chatbro';

	protected function __construct($backend)
	{
		parent::__construct($backend);
	}

	public static function get_store_id()
	{
		$store_id = self::get_backend()->store_id;

		if ($store_id === null || $store_id === '') {
			throw new CbroStoreNotFound('Undefined');
		}

		return (int) self::getDb()->escapeField($store_id);
	}

	public static function remote_post($uri, $args)
	{
		$curl_options = [];

		if (isset($args['headers'])) {
			$newHeaders = array_map(
				function ($key, $value) {
					return $key . ': ' . $value;
				},
				array_keys($args['headers']),
				$args['headers']
			);

			// Обновление исходного массива
			$args['headers'] = $newHeaders;

			$curl_options[CURLOPT_HTTPHEADER] = $args['headers'];
		}

		if (isset($args['body'])) {
			$curl_options[CURLOPT_POST] = true;
			$curl_options[CURLOPT_POSTFIELDS] = $args['body'];
		}

		$curl_options[CURLOPT_URL] = $uri;
		$curl_options[CURLOPT_RETURNTRANSFER] = true;
		$curl_options[CURLOPT_SSL_VERIFYHOST] = false;
		$curl_options[CURLOPT_SSL_VERIFYPEER] = false;

		$ch = curl_init();
		curl_setopt_array($ch, $curl_options);
		$result = curl_exec($ch);
		$info = curl_getinfo($ch);
		curl_close($ch);

		return [
			'result' => $result,
			'info' => $info,
		];
	}

	public static function remote_retrieve_response_code($response)
	{
		return isset($response['info']['http_code'])
			? (int) $response['info']['http_code']
			: 500;
	}

	public static function remote_retrieve_body($response)
	{
		return isset($response['result']) ? $response['result'] : null;
	}

	public static function is_error($response)
	{
		return (int) $response['info']['http_code'] >= 404;
	}

	public static function delete_access_token()
	{
		self::delete_option('chatbro_access_token');
	}

	public static function delete_refresh_token()
	{
		self::delete_option('chatbro_refresh_token');
	}

	public static function set_access_token($access_token)
	{
		return self::update_option('chatbro_access_token', $access_token);
	}

	public static function set_refresh_token($refresh_token)
	{
		return self::update_option('chatbro_refresh_token', $refresh_token);
	}

	public static function get_access_token()
	{
		return self::get_option('chatbro_access_token');
	}

	public static function get_refresh_token()
	{
		return self::get_option('chatbro_refresh_token');
	}

	private static function getDb()
	{
		return self::get_backend()->getDb();
	}

	public static function get_store($store_id)
	{
		$query = self::getDb()->select(self::$tableName, 'cb')
			->fields('cb')
			->condition('store_id', $store_id)
			->execute();

		return $query->fetchAssoc();
	}

	public static function reg_store()
	{
		$store_id = self::get_store_id();
		$databaseData = null;

		try {
			$query = self::getDb()->select(self::$tableName, 'cb')
				->fields('cb')
				->condition('store_id', $store_id)
				->execute();

			$databaseData = $query->fetchAssoc();
		} catch (Exception $e) {
			throw new CbroStoreNotFound(self::get_store_id());
		}

		if (!empty($databaseData)) {
			return;
		}

		try {
			$insert_values = [
				'chatbro_chat_guid' => CbroUtils::gen_guid(),
				'chatbro_chat_secret' => CbroUtils::gen_secret(),
				'store_id' => $store_id,
			];

			self::getDb()->insert(self::$tableName)
				->fields($insert_values)
				->execute();
		} catch (Exception $e) {
			throw new CbroStoreNotFound($store_id);
		}
	}

	public static function add_option($name, $value)
	{
		return self::update_option($name, $value);
	}

	public static function update_option($name, $value)
	{
		$store_id = self::get_store_id();

		try {
			self::getDb()->update(self::$tableName)
				->fields([
					$name => $value,
					'updated_at' => self::getDb()->query("SELECT NOW()")->fetchField(),
				])
				->condition('store_id', $store_id)
				->execute();
		} catch (Exception $e) {
			// throw new CbroStoreNotFound($updateSql);
			throw new CbroStoreNotFound(self::get_store_id());
		}
	}

	public static function get_option($name, $defaultValue = null)
	{
		$store_id = self::get_store_id();
		$store = self::get_store($store_id);

		if (empty($store)) {
			throw new CbroStoreNotFound(self::get_store_id());
		}

		if (!isset($store[$name]) || is_null($store[$name])) {
			return $defaultValue;
		}

		return $store[$name];
	}

	public static function delete_option($name)
	{
		$store_id = self::get_store_id();
		$store = self::get_store($store_id);

		if (empty($store)) {
			throw new CbroStoreNotFound(self::get_store_id());
		}

		if (isset($store[$name])) {
			return self::update_option($name, 'NULL');
		}

		return false;
	}

	public static function delete_store()
	{
	}

	public static function generated_scripts()
	{
	}
}
